/*
*
* Gtk+1.2-Gtk+2.x  Utility routines
* Copyright (C) 2002-2005 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

/********************* About and license file viewer Widget *******************************/

/* Needs TextCommon.c */
#include <gtk/gtk.h>
#include <stdlib.h>

#ifdef USE_GTK_2_0
#ifdef USE_GTK_2_4

static GtkActionEntry about_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "OptionsMenu",NULL,		"_"STR_OPTIONS,		NULL,				NULL,		NULL},	
};

static GtkToggleActionEntry about_check_entries[] = {
{	
	"Wrap", NULL,								/* name, stock id */
	"_"STR_WRAP, SHC_OPTIONS_WRAP,				/* label, accelerator */     
	NULL,										/* tooltip */
	G_CALLBACK (toggle_wrap_text),				/* callback */ 
	FALSE },									/* is_active */
};

static const gchar *about_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='Wrap'/>"
"    </menu>"
"  </menubar>"
"</ui>";

struct MenuEntry about_menu_entries[] = {
{ about_entries,	NULL,					NULL,	G_N_ELEMENTS (about_entries),			0, NULL},
{ NULL,				about_check_entries,	NULL,	G_N_ELEMENTS (about_check_entries),		0, NULL},
{ NULL,				NULL,					NULL,	0,										0, NULL},
/* Set last to NULL */ 
};

#else

static GtkItemFactoryEntry about_menu_items[] = {
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>",		NULL},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>",	NULL},
	{ NULL,							NULL,				NULL,					0,	NULL,			NULL},
	 /* Set last item to NULL */
};
#endif /* USE_GTK_2_4 */
#else

static GtkItemFactoryEntry about_menu_items[] = {
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>"		},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>"	},
	{ NULL,							NULL,				NULL,					0,	NULL			},
};
#endif

void license_callback(GtkWidget *widget, int *ptr_status)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	
	if(*ptr_status==0)
		SET2(*ptr_status,1);
	else
		SET2(*ptr_status,0);
}

int about_window_create(char * title, int x, int y, int border, char *labeltext, FILE *fd, int first_time)
{
	GtkWidget	*menubar;
	GtkWidget	*frame;
	GtkWidget	*label;
	GtkWidget	*bbox;
	GtkWidget	*check_button;
	GtkWidget	*button;
	int *ptr_status;
	int status=0;
	struct TextBuf *Tvdata;

	DBG_FUNCTION_NAME;

	/* Allocate memory for the data that is passed to the callbacks */
	Tvdata =(TextBuf*) xmalloc (sizeof (TextBuf));
	Tvdata->sourcefileinfo.st_mtime=0;
	Tvdata->sourcefileinfo.st_mode=0;
	Tvdata->text_modified=0;
	Tvdata->path=NULL;	
	ptr_status=&status;
	
	/* Make the gui first  */
	Tvdata->window = window_create(x, y, border, title, NULL, NULL, MODAL);
	
	Tvdata->table = table_create( 20, 18, Tvdata->window , 0, NOT_HOMOGENEUS);
#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (Tvdata->window), about_menu_entries, about_ui_info, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#else	
	menubar = main_menu_create(GTK_WIDGET (Tvdata->window), (GtkItemFactoryEntry *) &about_menu_items, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#endif	
	#ifdef USE_GTK_2_0
	frame = frame_create_on_table( NULL, Tvdata->table, 0, 20, 1, 4);
	label = label_create_on_table(labeltext, Tvdata->table, 0, 19, 1, 4);
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
	Tvdata->text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, Tvdata->table,  0, 20, 4, 16);
	#else
	frame = frame_create_on_table( NULL, Tvdata->table, 0, 20, 1, 6);
	label = label_create_on_table(labeltext, Tvdata->table, 0, 20, 1, 6);
	Tvdata->text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, Tvdata->table,  0, 19, 6, 16);
	#endif

	bbox = default_button_box_create_on_table( HORIZONTAL, Tvdata->table, 7, 20, 16, 18);
	button = button_create_in_container(STR_OK, NULL, NULL, bbox);
	
	if(first_time==FIRST_TIME) {
		#ifdef USE_GTK_2_0	
		check_button = check_create_on_table(STR_I_AGREE_DONT_SHOW_IT_AGAIN, GTK_SIGNAL_FUNC(license_callback), ptr_status,
											 INACTIVE, Tvdata->table, 0, 3, 17, 18);
		#else
		check_button = check_create_on_table(STR_I_AGREE_DONT_SHOW_IT_AGAIN, GTK_SIGNAL_FUNC(license_callback), ptr_status,
											 INACTIVE, Tvdata->table, 0, 7, 17, 18);
		#endif
	}

	gtk_widget_show_all(Tvdata->window);
	
	/* To speed up things a little at app startup */
	while(gtk_events_pending())
		gtk_main_iteration();
		
	/* Read from fd */
	if(fd!=NULL)
		read_from_fd_and_display_text(Tvdata->text, fd, 0, NULL);
	else
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_OPEN_FILE);
		
	/* Connect a callback to the close buttons of the viewer window */	
	signal_handlers_destroy(GTK_OBJECT(Tvdata->window));
	signal_connect(GTK_OBJECT(Tvdata->window), "delete_event", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);	
	signal_handlers_destroy(GTK_OBJECT(button));
	signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC (window_close), Tvdata->window);	
	Tvdata->window=main_loop(Tvdata->window);
	Tvdata=xfree(Tvdata);
	debug(err_msg("%s: done", __FUNCTION__));
	return status;
}

/*************** End of  About and license file viewer Widget ********************************/
