/*
*
* Gtk+1.2-Gtk+2.x  Utility routines
* Copyright (C) 2002-2005 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>

#ifdef USE_GTK_2_0
#ifdef USE_GTK_2_4
void toggle_wrap_text (GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));

	/*
	GTK_WRAP_NONE,
	GTK_WRAP_CHAR,
	GTK_WRAP_WORD
	*/
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(tdata->text),
								(gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(tdata->text))!=GTK_WRAP_NONE)?
								GTK_WRAP_NONE:
								GTK_WRAP_WORD);
}
#else /* USE_GTK_2_0 */
void toggle_wrap_text(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	
	assert(widget!=NULL);
	assert(tdata->text!=NULL);
	
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(tdata->text),
								(gtk_text_view_get_wrap_mode(GTK_TEXT_VIEW(tdata->text))!=GTK_WRAP_NONE)?
								GTK_WRAP_NONE:
								GTK_WRAP_WORD);
	callback_action=0;
}
#endif	
#else /* GTK_1_2 */
void toggle_wrap_text(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	
	assert(widget!=NULL);
	assert(tdata->text!=NULL);
	
	gtk_text_set_line_wrap(GTK_TEXT(tdata->text), (GTK_TEXT(tdata->text)->line_wrap==TRUE)?FALSE:TRUE);
	gtk_text_set_word_wrap(GTK_TEXT(tdata->text), (GTK_TEXT(tdata->text)->word_wrap==TRUE)?FALSE:TRUE);

	callback_action=0;
}
#endif

FILE *open_file(char *filename )
{
	/*   Inspired by files.c of nano editor 1.2.2                            *
	*                                                                        *
	*   Copyright (C) 1999-2003 Chris Allegretta GPL                         */

	int fd;
	FILE *file=NULL;
	struct stat fileinfo;

	DBG_FUNCTION_NAME;

	/* We ignore NULL silently as this should have been checked before calling open_file() */
	if(filename==NULL) {
		return NULL;
	}
	
	if (filename[0] == '\0' || stat(filename, &fileinfo) == -1) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_UNABLE_TO_STAT_S_S, filename, strerror(errno));
	}
	else if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) || S_ISBLK(fileinfo.st_mode)) {
		/* Don't open character or block files.  Sorry, /dev/sndstat! */
		if(S_ISDIR(fileinfo.st_mode)) {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_S_IS_A_DIR, filename);
		} else {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_FILE_S_IS_A_DEVICE_FILE, filename);
		}
	} else if ((fd = open(filename, O_RDONLY)) == -1) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, "%s: %s", filename, strerror(errno));
	} else {
		/* File is A-OK */
		/* Binary for our own line-end munging */
		/*The mode string can also include the letter  ``b''  either  as  a  last
		character  or  as a character between the characters in any of the two-
		character strings described above.  This is strictly for  compatibility
		with  ANSI  X3.159-1989  (``ANSI  C'')  and has no effect; the ``b'' is
		ignored on all POSIX conforming systems, including Linux.*/
		file = fdopen(fd, "rb");
		if (file == NULL) {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1,  STR_ERROR_READING_FILE_S_S, filename, strerror(errno));
			close(fd);
			file=NULL;
		}
	}
	/* end of nano code */
	return file;
}

#ifdef USE_GTK_2_0
int read_and_convert(char * temp_ptr, GtkTextBuffer *buffer, GtkTextIter iter)
{
	char * utf8_temp=NULL;

	DBG_FUNCTION_NAME;
		
	/* Convert to UTF-8 */
	if((utf8_temp=g_locale_to_utf8(temp_ptr, -1,NULL,NULL,NULL))==NULL)
	{
		dialog_window(STR_ALERT, ONE_BUTTON, NULL, 1,  STR_CONVERSION_TO_UTF8_FAILED);
		return 1;
	}
	if(GTK_IS_TEXT_BUFFER (buffer))
	{
		gtk_text_buffer_get_end_iter(buffer, &iter);
		gtk_text_buffer_insert_with_tags_by_name (buffer, &iter, utf8_temp, -1, "monospace", NULL);
		utf8_temp=xfree(utf8_temp);
		while (gtk_events_pending ())
			gtk_main_iteration();
		return 0;
	}
	utf8_temp=xfree(utf8_temp);
	return 1;
}
#endif

#ifdef USE_GTK_2_0
void read_from_fd_and_display_text(GtkWidget *text, FILE *fd, unsigned long filesize, gdouble *progress)
{
	GtkTextBuffer	*buffer=NULL;
	GtkTextIter		iter;
	char *temp_ptr=NULL;
	float step=0.0;
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	
	if(filesize>0 && fd!=stdin)
		step=(float)(1/((float)filesize/DEF_BUFFER_SIZE));
	
	debug(err_msg("filesize %u iter %u step %f",filesize, filesize/DEF_BUFFER_SIZE, step));
		
	while((temp_ptr=xfgetc_nbuf(fd, DEF_BUFFER_SIZE))!=NULL)
	{
		while (gtk_events_pending ())
			gtk_main_iteration();
		if(read_and_convert(temp_ptr, buffer, iter)==1)
			break;
		if(progress!=NULL)
			*progress+=step;
		temp_ptr=xfree(temp_ptr);
	}
	temp_ptr=xfree(temp_ptr);
	return;
}
#else
void read_from_fd_and_display_text(GtkWidget *text, FILE *fd, unsigned long filesize, gdouble *progress)
{
	GdkFont *fixed_font;
	char *temp_ptr=NULL;
	float step=0.0;
	
	/* Load a fixed font */
	fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-130-*-*-*-*-*-*");

	if(GTK_IS_TEXT(text))
		gtk_text_freeze(GTK_TEXT(text));
	
	if(filesize > 0 && fd!=stdin)
		step=(float)(1/((float)filesize/DEF_BUFFER_SIZE));
	
	debug(err_msg("filesize %u iter %u step %f",filesize, filesize/DEF_BUFFER_SIZE, step));
	
	while((temp_ptr=xfgetc_nbuf(fd, DEF_BUFFER_SIZE))!=NULL)
	{
		if(progress!=NULL)
			*progress+=step;
		if(GTK_IS_WIDGET(text)) {
			gtk_text_insert(GTK_TEXT(text), fixed_font, &text->style->black, NULL, temp_ptr, -1);
		} else {
			temp_ptr=xfree(temp_ptr);
			break;
		}
	
		while (gtk_events_pending ())
			gtk_main_iteration();
		
		temp_ptr=xfree(temp_ptr);
	}
	
	if(GTK_IS_TEXT(text))
		gtk_text_thaw(GTK_TEXT(text));
}
#endif
