/*
*
* Gtk+1.2-Gtk+2.x  Utility routines
* Copyright (C) 2002-2005 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <strings.h>

struct TextBuf *Tdata;

/********************* Text file edit Widget *******************************/

void save_file(char *filename, GtkWidget *text)
{
	/*   Inspired by files.c of nano editor 1.2.2                            *
	*                                                                        *
	*   Copyright (C) 1999-2003 Chris Allegretta GPL                         *
	*   realeased under the terms of the GPL                                 */

	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		start;
	GtkTextIter		end;
	#endif
	char *data=NULL;
	int flag;
	int fd;
	FILE *file=NULL;
	struct stat fileinfo;

	debug(err_msg("%s(): %s", __FUNCTION__, filename));

	if (filename[0] == '\0' || filename == NULL ) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_NO_FILE_NAME);
		return;
	}

	if (stat(filename, &fileinfo) == 0) {
		/* File exists */
		if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) || S_ISBLK(fileinfo.st_mode)) {
			if(S_ISDIR(fileinfo.st_mode)) {
				dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_S_IS_A_DIR, filename);
			} else {
				dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_FILE_S_IS_A_DEVICE_FILE, filename);
			}
			return;
		}
		flag=O_WRONLY|  O_TRUNC ;
	} else {
		flag= O_WRONLY| O_CREAT;
	}

	if ((fd = open(filename,  flag )) == -1) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, "%s: %s", filename, strerror(errno));
		return;
	}

	/* File is A-OK */
	/*The mode string can also include the letter  ``b''  either  as  a  last
	character  or  as a character between the characters in any of the two-
	character strings described above.  This is strictly for  compatibility
	with  ANSI  X3.159-1989  (``ANSI  C'')  and has no effect; the ``b'' is
	ignored on all POSIX conforming systems, including Linux.*/
	if ((file = fdopen(fd, "wb"))== NULL) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_ERROR_WRITING_FILE_S_S, filename, strerror(errno));
		close(fd);
		return;
	}
	#ifdef USE_GTK_2_0
	buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	data=g_locale_from_utf8(gtk_text_buffer_get_text(buffer, &start, &end,/*include_hidden_chars*/ FALSE), -1, NULL, NULL, NULL);
	#else
	data=gtk_editable_get_chars( GTK_EDITABLE(text) , 0, -1);
	#endif
	if(data==NULL) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SAVE_S_UNKNOWN_ERROR, filename);
		fclose(file);
		return;
	}
	fprintf(file,"%s", data);
	data=xfree(data);
	fclose(file);
}

#ifdef USE_GTK_2_0
#ifdef USE_GTK_2_4
static GtkActionEntry fake_text_edit_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		"_"STR_FILE,		NULL,				NULL,		NULL},
	{ "OptionsMenu",NULL,		"_"STR_OPTIONS,		NULL,				NULL,		NULL},	
	{ "HelpMenu",	NULL,		"_"STR_HELP,		NULL,				NULL,		NULL},
	{ "New",		NULL,		"_"STR_NEW, 		SHC_FILE_NEW,		NULL,		G_CALLBACK (gtk_true) },   
	{ "Open",		NULL,		"_"STR_OPEN,		SHC_FILE_OPEN,		NULL,		G_CALLBACK (gtk_true) }, 
	{ "Save",		NULL,		"_"STR_SAVE,		SHC_FILE_SAVE,		NULL,		G_CALLBACK (gtk_true) },
	{ "SaveAs",		NULL,		STR_SAVE__AS,		SHC_FILE_SAVE_AS,	NULL,		G_CALLBACK (gtk_true) },
	{ "Quit",		NULL,		"_"STR_QUIT,		SHC_FILE_QUIT,		NULL,		G_CALLBACK (gtk_true) },
	{ "About",		NULL,		STR_ABOUT,			NULL,				NULL,		G_CALLBACK (about_callback) },
};

static GtkActionEntry text_edit_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		"_"STR_FILE,		NULL,				NULL,		NULL},
	{ "OptionsMenu",NULL,		"_"STR_OPTIONS,		NULL,				NULL,		NULL},	
	{ "HelpMenu",	NULL,		"_"STR_HELP,		NULL,				NULL,		NULL},
	{ "New",		NULL,		"_"STR_NEW, 		SHC_FILE_NEW,		NULL,		G_CALLBACK (new_callback) },   
	{ "Open",		NULL,		"_"STR_OPEN,		SHC_FILE_OPEN,		NULL,		G_CALLBACK (open_callback) }, 
	{ "Save",		NULL,		"_"STR_SAVE,		SHC_FILE_SAVE,		NULL,		G_CALLBACK (save_callback) },
	{ "SaveAs",		NULL,		STR_SAVE__AS,		SHC_FILE_SAVE_AS,	NULL,		G_CALLBACK (save_as_callback) }, 
	{ "Quit",		NULL,		"_"STR_QUIT,		SHC_FILE_QUIT,		NULL,		G_CALLBACK (text_quit_callback) },
	{ "About",		NULL,		STR_ABOUT,			NULL,				NULL,		G_CALLBACK (about_callback) },
};

static GtkToggleActionEntry text_edit_check_entries[] = {
{	
	"Wrap", NULL,						/* name, stock id */
	"_"STR_WRAP, SHC_OPTIONS_WRAP,		/* label, accelerator */     
	NULL,								/* tooltip */
	G_CALLBACK (toggle_wrap_text),		/* callback */ 
	FALSE },							/* is_active */
};

static const gchar *text_edit_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='New'/>"
"      <menuitem action='Open'/>"
"      <menuitem action='Save'/>"
"      <menuitem action='SaveAs'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='Wrap'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

struct MenuEntry fake_text_edit_menu_entries[] = {
{ fake_text_edit_entries,	NULL,					NULL,			G_N_ELEMENTS (fake_text_edit_entries),		0, NULL},
{ NULL,						text_edit_check_entries,NULL,			G_N_ELEMENTS (text_edit_check_entries),		0, NULL},
{ NULL,						NULL,					NULL,			0,											0, NULL},
/* Set last to NULL */ 
};

struct MenuEntry text_edit_menu_entries[] = {
{ text_edit_entries,	NULL,						NULL,			G_N_ELEMENTS (text_edit_entries),			0, NULL},
{ NULL,					text_edit_check_entries,	NULL,			G_N_ELEMENTS (text_edit_check_entries),		0, NULL},
{ NULL,					NULL,						NULL,			0,											0, NULL},
/* Set last to NULL */ 
};

#else
static GtkItemFactoryEntry fake_text_edit_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,								0,	"<Branch>",		NULL},
	{ "/"STR_FILE"/_"STR_NEW,		SHC_FILE_NEW,		NULL,								0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_OPEN,		SHC_FILE_OPEN,		NULL,								0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_SAVE,		SHC_FILE_SAVE,		NULL,								0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_SAVE__AS,	SHC_FILE_SAVE_AS,	NULL,								0,	NULL,			NULL},
	{ "/"STR_FILE"/sep1",			NULL,				NULL,								0, "<Separator>",	NULL},	
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		NULL,								0,	NULL,			NULL},
	{ "/_"STR_OPTIONS,				NULL,				NULL,								0,	"<Branch>",		NULL},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,					0,	"<CheckItem>",	NULL},
	{ "/_"STR_HELP,					NULL,				NULL,								0,	"<LastBranch>",	NULL},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,						0,	NULL,			NULL},
	{ NULL,							NULL,				NULL,								0,	NULL,			NULL},
	 /* Set last item to NULL */
};

static GtkItemFactoryEntry text_edit_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,								0,	"<Branch>",		NULL},
	{ "/"STR_FILE"/_"STR_NEW,		SHC_FILE_NEW,		new_callback,						0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_OPEN,		SHC_FILE_OPEN,		open_callback,						0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_SAVE,		SHC_FILE_SAVE,		save_callback,						0,	NULL,			NULL},
	{ "/"STR_FILE"/_"STR_SAVE__AS,	SHC_FILE_SAVE_AS,	save_as_callback,					0,	NULL,			NULL},
	{ "/"STR_FILE"/sep1",			NULL,				NULL,								0, "<Separator>",	NULL},	
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		text_quit_callback,					0,	NULL,			NULL},
	{ "/_"STR_OPTIONS,				NULL,				NULL,								0,	"<Branch>",		NULL},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,					0,	"<CheckItem>",	NULL},
	{ "/_"STR_HELP,					NULL,				NULL,								0,	"<LastBranch>",	NULL},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,						0,	NULL,			NULL},
	{ NULL,							NULL,				NULL,								0,	NULL,			NULL},
	 /* Set last item to NULL */
};
#endif /* USE_GTK_2_4 */
#else
static GtkItemFactoryEntry fake_text_edit_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,								0,	"<Branch>"		},
	{ "/"STR_FILE"/_"STR_NEW,		SHC_FILE_NEW,		NULL,								0,	NULL			},
	{ "/"STR_FILE"/_"STR_OPEN,		SHC_FILE_OPEN,		NULL,								0,	NULL			},
	{ "/"STR_FILE"/_"STR_SAVE,		SHC_FILE_SAVE,		NULL,								0,	NULL			},
	{ "/"STR_FILE"/_"STR_SAVE__AS,	SHC_FILE_SAVE_AS,	NULL,								0,	NULL			},
	{ "/"STR_FILE"/sep1",			NULL,				NULL,								0, "<Separator>"	},	
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		NULL,								0,	NULL			},
	{ "/_"STR_OPTIONS,				NULL,				NULL,								0,	"<Branch>"		},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,					0,	"<CheckItem>"	},
	{ "/_"STR_HELP,					NULL,				NULL,								0,	"<LastBranch>"	},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,						0,	NULL			},
	{ NULL,							NULL,				NULL,								0,	NULL			},
};

static GtkItemFactoryEntry text_edit_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,								0,	"<Branch>"		},
	{ "/"STR_FILE"/_"STR_NEW,		SHC_FILE_NEW,		new_callback,						0,	NULL			},
	{ "/"STR_FILE"/_"STR_OPEN,		SHC_FILE_OPEN,		open_callback,						0,	NULL			},
	{ "/"STR_FILE"/_"STR_SAVE,		SHC_FILE_SAVE,		save_callback,						0,	NULL			},
	{ "/"STR_FILE"/_"STR_SAVE__AS,	SHC_FILE_SAVE_AS,	save_as_callback,					0,	NULL			},
	{ "/"STR_FILE"/sep1",			NULL,				NULL,								0, "<Separator>"	},	
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		text_quit_callback,					0,	NULL			},
	{ "/_"STR_OPTIONS,				NULL,				NULL,								0,	"<Branch>"		},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,					0,	"<CheckItem>"	},
	{ "/_"STR_HELP,					NULL,				NULL,								0,	"<LastBranch>"	},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,						0,	NULL			},
	{ NULL,							NULL,				NULL,								0,	NULL			},
};
#endif

void clear_the_buffer(GtkWidget *text, int *text_modified)
{
	#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		start,
					end;

	DBG_FUNCTION_NAME;
	buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
	#else

	DBG_FUNCTION_NAME;
	gtk_text_backward_delete(GTK_TEXT(text) ,gtk_text_get_length(GTK_TEXT(text)));
	#endif
	RESET(*text_modified);
}

void text_changed_callback(GtkWidget *widget, int *ptr_text_modified)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);

	SET(*ptr_text_modified);
}

int overwrite(char *msg_str)
{
	DBG_FUNCTION_NAME;

	if(	dialog_window(STR_WARNING, TWO_BUTTONS, NULL, 2, "%s", msg_str)==1) {
		return 1;
	}
	return 0;
}

void save_common( TextBuf *tdata)
{
	mode_t mask;
	struct stat destfileinfo;
		
	/* File exists */
	if(stat(tdata->path, &destfileinfo)==0) {
		if(overwrite(STR_DEST_FILE_EXIST)==1) {
			return;
		}
		mask = destfileinfo.st_mode;
		/* File is newer */
		if(tdata->sourcefileinfo.st_mtime!=0 && tdata->sourcefileinfo.st_mtime > destfileinfo.st_mtime ) {
			if(overwrite(STR_DEST_FILE_NEWER)==1)
				return;
		}
	} else {
		if(tdata->sourcefileinfo.st_mode!=0) {
			mask=tdata->sourcefileinfo.st_mode;
		} else {
			mask = 0644; /* Use default file permissions */
		}
	}
	/* write file*/ 
	save_file(tdata->path, tdata->text);
	tdata->text_modified=0;
	/* Set permission */
	if (chmod(tdata->path, mask) == -1) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SET_FILE_PERMISSIONS_ON_S_S, tdata->path, strerror(errno));
	}
	/* Set ownership is not needed as it is set to that of the current process */
	
	gtk_window_set_title(GTK_WINDOW(tdata->window), tdata->path);
}

#ifdef USE_GTK_2_4
void save_as_callback(GtkAction *menu_action, TextBuf *tdata)
{
	char *new_path;
	
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	if((new_path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
		return;
	}
	tdata->path=xfree(tdata->path);
	tdata->path=new_path;
	save_common(tdata);
}
#else
void save_as_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	char *new_path;
	
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	if((new_path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
		return;
	}
	tdata->path=xfree(tdata->path);
	tdata->path=new_path;
	save_common(tdata);
	
	callback_action=0;
}
#endif

#ifdef USE_GTK_2_4
void save_callback(GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	if(tdata->path==NULL) {
	/* No file name: ask for it */ 
		if((tdata->path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
			return;
		}
	}
	save_common(tdata);
}
#else
void save_callback( TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	if(tdata->path==NULL) {
	/* No file name: ask for it */ 
		if((tdata->path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
			return;
		}
	}
	save_common(tdata);
	
	callback_action=0;
}
#endif

void open_new_common( TextBuf *tdata)
{
	if(tdata->text_modified==TEXT_MODIFIED){
		if(tdata->path==NULL) {
		/* No file name: ask for it */ 
			if((tdata->path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
				return;
			}
		}	
		save_common(tdata);
	}
	
	RESET(tdata->sourcefileinfo.st_mtime);
	RESET(tdata->sourcefileinfo.st_mode);
	gtk_window_set_title(GTK_WINDOW(tdata->window), STR_UNTITLED);
	clear_the_buffer(tdata->text, &tdata->text_modified);
	tdata->path=xfree(tdata->path);
	tdata->text_modified=TEXT_NOT_MODIFIED;
	tdata->sourcefileinfo.st_mtime=0;
	tdata->sourcefileinfo.st_mode=0;
}

#ifdef USE_GTK_2_4
void new_callback(GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	open_new_common(tdata);
}
#else
void new_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	open_new_common(tdata);
	
	callback_action=0;
}
#endif

void open_common(TextBuf *tdata)
{
	GtkWidget	*menubar;
	ProgressData * pdata=NULL;
	FILE *file=NULL;
#ifdef USE_GTK_2_0
	GtkTextIter		iter;
#endif
	DBG_FUNCTION_NAME;

	/* Add a menubar  with disabled quit menu as long as we have not loaded the text in the viewer */ 
#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (tdata->window), fake_text_edit_menu_entries, text_edit_ui_info, tdata);
	gtk_table_attach_defaults (GTK_TABLE(tdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#else	
	menubar = main_menu_create(tdata->window, (GtkItemFactoryEntry *) &fake_text_edit_menu_items, tdata);
	gtk_table_attach_defaults (GTK_TABLE(tdata->table),GTK_WIDGET(menubar), 0, 20, 0, 2);
#endif	
	/* Connect a fake-callback to the close button of the viewer window */	
	signal_handlers_destroy(GTK_OBJECT(tdata->window));
	signal_connect(GTK_OBJECT(tdata->window), "delete_event", GTK_SIGNAL_FUNC (gtk_true), NULL);	
	gtk_widget_show_all(GTK_WIDGET(tdata->window));
		
	#ifdef USE_GTK_2_0
	gtk_text_buffer_get_iter_at_offset (gtk_text_view_get_buffer (GTK_TEXT_VIEW (tdata->text)), &iter, 0);
	#endif
	/* During file reading delete_event and quit signals are ignored */
	if(tdata->path!=NULL && stat(tdata->path, &tdata->sourcefileinfo)==0) {
		if((file=open_file(tdata->path))!=NULL) {
			/* Errors in opening the file are showed by open_file itself */
			gtk_window_set_title(GTK_WINDOW(tdata->window), tdata->path);
			pdata=progressbar_create_on_table(PULSE, USE_VALUE, tdata->table, 0, 10, 20, 21 );
			gtk_widget_show(pdata->pbar);
			read_from_fd_and_display_text(tdata->text, file, tdata->sourcefileinfo.st_size, &pdata->value);
			xfclose_nostdin(file);
			progress_destroy(pdata);
		} else {
			/* Some error occurred in opening the file: S_ISDIR, S_ISCHR, S_ISBLK, so create create an untitled empty edit window */
			gtk_window_set_title(GTK_WINDOW(tdata->window), STR_UNTITLED );
		}
	} else {
		/*create an empty edit window titled or untitled */
		gtk_window_set_title(GTK_WINDOW(tdata->window),(tdata->path!=NULL) ? tdata->path : STR_UNTITLED);
	}
	/* Now add the REAL menubar so that we can interact with the viewer */ 
	gtk_widget_destroy(menubar);
#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (tdata->window), text_edit_menu_entries, text_edit_ui_info, tdata);
	gtk_table_attach_defaults (GTK_TABLE(tdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#else	
	menubar = main_menu_create(tdata->window, (GtkItemFactoryEntry *) &text_edit_menu_items, tdata);
	gtk_table_attach_defaults (GTK_TABLE(tdata->table),GTK_WIDGET(menubar), 0, 20, 0, 2);
#endif
	gtk_widget_show(GTK_WIDGET(menubar));
	/* Connect a callback to the close button of the viewer window */	
	signal_handlers_destroy(GTK_OBJECT(tdata->window));
	/* We don't pass tdata here to the GTK_SIGNEL_FUNC as it is not updated correctly and breaks the quit_common() code */
	signal_connect(GTK_OBJECT(tdata->window), "delete_event", GTK_SIGNAL_FUNC (text_window_quit_callback), NULL);
	SET2(tdata->text_modified,TEXT_NOT_MODIFIED);
}

#ifdef USE_GTK_2_4
void open_callback(GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	
	open_new_common(tdata);
	if((tdata->path=file_selector( STR_OPEN":", SHOW_FILE_OP_BUTTONS))==NULL) {
		return;
	}
	open_common(tdata);
}
#else
void open_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	
	assert(widget!=NULL);
	
	open_new_common(tdata);
	if((tdata->path=file_selector( STR_OPEN":", SHOW_FILE_OP_BUTTONS))==NULL) {
		return;
	}
	open_common(tdata);
	
	callback_action=0;
}
#endif

void quit_common(TextBuf *tdata)
{	
	DBG_FUNCTION_NAME;
		
	if(tdata->text_modified==TEXT_MODIFIED &&  
		dialog_window(STR_SAVE_FILE, TWO_BUTTONS, NULL, 2, STR_FILE_MODIFIED_WANT_TO_SAVE)==0) {
		if(tdata->path==NULL) {
			/* No file name: ask for it */ 
			if((tdata->path=file_selector( STR_SAVE_AS":", SHOW_FILE_OP_BUTTONS))==NULL) {
				return;
			}
		}	
		save_common(tdata);
	}
	gtk_main_quit();
	if(tdata->window!=NULL && GTK_IS_WIDGET(tdata->window) && GTK_WIDGET_TOPLEVEL(tdata->window)) {
		gtk_widget_destroy(tdata->window);
		gtk_widget_destroyed(tdata->window, &tdata->window);
	}
}

#ifdef USE_GTK_2_4
void text_quit_callback(GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	quit_common(tdata);
}
#else
void text_quit_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	
	quit_common(tdata);
	
	callback_action=0;
}
#endif

void text_window_quit_callback(GtkWidget *widget, gpointer data)
{
	DBG_FUNCTION_NAME;
	
	assert(widget!=NULL);
	/* We don't pass tdata through the callback here as it is not updated correctly
	and breaks quit common */
	quit_common(Tdata);

	/* to shut a compiler warning */
	data=NULL;	
}

void textedit_window_create(char * title, int x, int y, int border, char *filename)
{
	DBG_FUNCTION_NAME;

	/* Allocate memory for the data that is passed to the callbacks */
	Tdata =(TextBuf*) xmalloc (sizeof (TextBuf));
	
	/* Make our own copy of filname so we can free it later */
	if(filename!=NULL){
		Tdata->path=xstrdup(filename);
	} else {
		Tdata->path=NULL;
	}
	
	Tdata->window= window_create( x, y, border, title,  NULL, NULL,  MODAL);
	Tdata->sourcefileinfo.st_mtime=0;
	Tdata->sourcefileinfo.st_mode=0;
	Tdata->text_modified=0;
	Tdata->table = table_create( 20, 21,GTK_WIDGET(Tdata->window), 0, HOMOGENEUS);
#ifdef USE_GTK_2_0
	Tdata->text = textpad_create_on_table(EDITABLE, GTK_SIGNAL_FUNC(text_changed_callback), &Tdata->text_modified, NO_WRAP, SCROLL,
									Tdata->table,  0, 20, 1, 20);
#else
	Tdata->text = textpad_create_on_table(EDITABLE, GTK_SIGNAL_FUNC(text_changed_callback), &Tdata->text_modified, NO_WRAP, SCROLL,
									Tdata->table,  0, 20, 2, 20);
#endif
	open_common(Tdata);
	Tdata->window=main_loop(Tdata->window);
	Tdata->path=xfree(Tdata->path);
	Tdata=xfree(Tdata);
	debug(err_msg("%s: done", __FUNCTION__));
	return;
}

/***************End of Text file edit Widget ********************************/
