/*
*
* Gtk+1.2-Gtk+2.x  Utility routines
* Copyright (C) 2002-2005 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <strings.h>

/********************* Text file View Widget *******************************/

#ifdef USE_GTK_2_0
#ifdef USE_GTK_2_4
static GtkActionEntry fake_text_view_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		"_"STR_FILE,		NULL,				NULL,		NULL},
	{ "OptionsMenu",NULL,		"_"STR_OPTIONS,		NULL,				NULL,		NULL},	
	{ "HelpMenu",	NULL,		"_"STR_HELP,		NULL,				NULL,		NULL},
	{ "Quit",		NULL,		"_"STR_QUIT,		SHC_FILE_QUIT,		NULL,		G_CALLBACK(gtk_true) },
	{ "About",		NULL,		STR_ABOUT,			NULL,				NULL,		G_CALLBACK(about_callback) },
};

static GtkActionEntry text_view_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		"_"STR_FILE,		NULL,				NULL,		NULL},
	{ "OptionsMenu",NULL,		"_"STR_OPTIONS,		NULL,				NULL,		NULL},	
	{ "HelpMenu",	NULL,		"_"STR_HELP,		NULL,				NULL,		NULL},
	{ "Quit",		NULL,		"_"STR_QUIT,		SHC_FILE_QUIT,		NULL,		G_CALLBACK(textview_quit_callback) },
	{ "About",		NULL,		STR_ABOUT,			NULL,				NULL,		G_CALLBACK(about_callback) },
};

static GtkToggleActionEntry text_view_check_entries[] = {
{	
	"Wrap", NULL,								/* name, stock id */
	"_"STR_WRAP, SHC_OPTIONS_WRAP,				/* label, accelerator */     
	NULL,										/* tooltip */
	G_CALLBACK (toggle_wrap_text),				/* callback */ 
	FALSE },									/* is_active */
};

static const gchar *text_view_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='Wrap'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

struct MenuEntry text_view_menu_entries[] = {
{ text_view_entries,	NULL,						NULL,	G_N_ELEMENTS (text_view_entries),			0, NULL},
{ NULL,					text_view_check_entries,	NULL,	G_N_ELEMENTS (text_view_check_entries),		0, NULL},
{ NULL,					NULL,						NULL,	0,											0, NULL},
/* Set last to NULL */ 
};

struct MenuEntry fake_text_view_menu_entries[] = {
{ fake_text_view_entries,	NULL,					NULL,	G_N_ELEMENTS (fake_text_view_entries),		0, NULL},
{ NULL,					text_view_check_entries,	NULL,	G_N_ELEMENTS (text_view_check_entries),		0, NULL},
{ NULL,					NULL,						NULL,	0,											0, NULL},
/* Set last to NULL */ 
};

#else
static GtkItemFactoryEntry fake_text_view_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,					0,	"<Branch>",		NULL},
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		NULL,					0,	NULL,			NULL},
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>",		NULL},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>",	NULL},
	{ "/_"STR_HELP,					NULL,				NULL,					0,	"<LastBranch>",	NULL},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,			0,	NULL,			NULL},
	{ NULL,							NULL,				NULL,					0,	NULL,			NULL},
	 /* Set last item to NULL */
};

static GtkItemFactoryEntry text_view_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,					0,	"<Branch>",		NULL},
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		textview_quit_callback,	0,	NULL,			NULL},
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>",		NULL},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>",	NULL},
	{ "/_"STR_HELP,					NULL,				NULL,					0,	"<LastBranch>",	NULL},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,			0,	NULL,			NULL},
	{ NULL,							NULL,				NULL,					0,	NULL,			NULL},
	 /* Set last item to NULL */
};
#endif /* USE_GTK_2_4 */
#else
static GtkItemFactoryEntry fake_text_view_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,					0,	"<Branch>"		},
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		NULL,					0,	NULL			},
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>"		},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>"	},
	{ "/_"STR_HELP,					NULL,				NULL,					0,	"<LastBranch>"	},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,			0,	NULL			},
	{ NULL,							NULL,				NULL,					0,	NULL			},
};

static GtkItemFactoryEntry text_view_menu_items[] = {
	{ "/_"STR_FILE,					NULL,				NULL,					0,	"<Branch>"		},
	{ "/"STR_FILE"/_"STR_QUIT,		SHC_FILE_QUIT,		textview_quit_callback,	0,	NULL			},
	{ "/_"STR_OPTIONS,				NULL,				NULL,					0,	"<Branch>"		},
	{ "/"STR_OPTIONS"/_"STR_WRAP,	SHC_OPTIONS_WRAP,	toggle_wrap_text,		0,	"<CheckItem>"	},
	{ "/_"STR_HELP,					NULL,				NULL,					0,	"<LastBranch>"	},
	{ "/_"STR_HELP"/"STR_ABOUT,		NULL,				about_callback,			0,	NULL			},
	{ NULL,							NULL,				NULL,					0,	NULL			},
};
#endif

#ifdef USE_GTK_2_4
void textview_quit_callback(GtkAction *menu_action, TextBuf *tdata)
{
	DBG_FUNCTION_NAME;
	debug(err_msg("Action \"%s\" activated", gtk_action_get_name (menu_action)));
	gtk_main_quit();
	gtk_widget_destroy(tdata->window);
}
#else
void textview_quit_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);
	
	gtk_main_quit();
	gtk_widget_destroy(tdata->window);	
	
	/* to shut a compiler warning */
	callback_action=0;
}
#endif

void textview_window_create(char * title, int x, int y, int border, char *filename, FILE *fd)
{
	GtkWidget	*menubar;
	ProgressData * pdata=NULL;
#ifdef USE_GTK_2_0
	GtkTextBuffer	*buffer;
	GtkTextIter		iter;
#endif
	char *path=NULL;
	FILE *file=NULL;
	struct TextBuf *Tvdata;
	struct stat sourcefileinfo;

	DBG_FUNCTION_NAME;

	/* Allocate memory for the data that is passed to the callbacks */
	Tvdata =(TextBuf*) xmalloc (sizeof (TextBuf));
	Tvdata->sourcefileinfo.st_mtime=0;
	Tvdata->sourcefileinfo.st_mode=0;
	Tvdata->text_modified=0;
	
	/* Make our own copy of filname so we can free it later */
	if(filename!=NULL) {
		Tvdata->path=xstrdup(filename);
	} else {
		Tvdata->path=NULL;
	}
	
	Tvdata->window = window_create( x, y, border, title,  NULL, NULL, MODAL);
	Tvdata->table = table_create( 20, 21,GTK_WIDGET(Tvdata->window), 0, HOMOGENEUS);
#ifdef USE_GTK_2_0
	Tvdata->text = textpad_create_on_table(NOT_EDITABLE, NULL, NULL, NO_WRAP, SCROLL, Tvdata->table,  0, 20, 1, 20);
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (Tvdata->text));
#else
	Tvdata->text = textpad_create_on_table(NOT_EDITABLE, NULL, NULL, NO_WRAP, SCROLL, Tvdata->table,  0, 20, 2, 20);
#endif
	/* Add a menubar  with disabled quit menu as long as we have not loaded the text in the viewer */ 
#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (Tvdata->window), fake_text_view_menu_entries, text_view_ui_info, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#else	
	menubar = main_menu_create(Tvdata->window, (GtkItemFactoryEntry *) &fake_text_view_menu_items, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 2);
#endif		
	gtk_widget_show_all(GTK_WIDGET(Tvdata->window));

	#ifdef USE_GTK_2_0
	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
	#endif

	/* As long as we load the files delete_event signals and quit callbacks are ignored */
	if (fd!=NULL) {
		/* This is needed mainly for reading from pipes.
			We must close the fd by ourselves with fclose or pclose */
		debug(err_msg("%s: using fd",__FUNCTION__));
		pdata=progressbar_create_on_table(ACTIVITY, NOT_USE_VALUE, Tvdata->table, 0, 10, 20, 21 );
		gtk_widget_show(pdata->pbar);
		read_from_fd_and_display_text(Tvdata->text, fd, 0, NULL);
	} else if (Tvdata->path!=NULL && stat(Tvdata->path, &sourcefileinfo)==0) {
		debug(err_msg("%s: using path: %s",__FUNCTION__, path));
		if((file=open_file(Tvdata->path))!=NULL) {
			/* Errors in opening the file are showed by open_file itself */
			pdata=progressbar_create_on_table(PULSE, USE_VALUE, Tvdata->table, 0, 10, 20, 21 );
			gtk_widget_show(pdata->pbar);
			read_from_fd_and_display_text(Tvdata->text, file, sourcefileinfo.st_size, &pdata->value);
			xfclose_nostdin(file);
		}
	} else {
		title="";
	}
	
	/* else show a blank textview window */

	/* Now add the REAL menubar so that we can interact with the viewer */ 
	gtk_widget_destroy(menubar);
#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (Tvdata->window), text_view_menu_entries, text_view_ui_info, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 1);
#else	
	menubar = main_menu_create(Tvdata->window, (GtkItemFactoryEntry *) &text_view_menu_items, Tvdata);
	gtk_table_attach_defaults (GTK_TABLE(Tvdata->table),GTK_WIDGET(menubar), 0, 20, 0, 2);
	gtk_widget_show(menubar);
#endif	
	
	if(Tvdata->path!=NULL)
		gtk_window_set_title(GTK_WINDOW(Tvdata->window), Tvdata->path);
	if(pdata)
		progress_destroy(pdata);

	/* Connect a callback to the close button of the viewer window */	
	signal_handlers_destroy(GTK_OBJECT(Tvdata->window));
	signal_connect(GTK_OBJECT(Tvdata->window), "delete_event", GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
	
	Tvdata->window=main_loop(Tvdata->window);
	Tvdata->path=xfree(Tvdata->path);
	Tvdata=xfree(Tvdata);
	debug(err_msg("%s: done", __FUNCTION__));
	return;
}

/***************End of Text file View Widget ********************************/
