/*
*
* XFPROT Copyright (C) 2002-2005 by Tito Ragusa <tito-wolit@tiscali.it>
*
* XFPROT is a graphical frontend for the F-Prot Antivirus
* for Linux Small Business Edition. It was based upon the
* XSTEP-3.5.1 library until version 0.19beta.
* Starting from version 0.20beta it is based upon Gtk-1.2.10.
* From version 1.0-rc2 it is also possible to use Gtk-2.x.x libraries.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <assert.h>

/* This is to avoid a compiler warning for `index` with -Wshadow and Gtk-+-1.2 libs */
#ifndef USE_GTK_2_0
	#ifdef IS_LINUX
		#undef  __USE_BSD
		#include <string.h>
		#define  __USE_BSD
	#endif
#else
	#include <string.h>
#endif

#ifdef NO_CORE_DUMPS
	#include <sys/prctl.h>
#endif

/* for checkboxes */
int	flag_nosub=0;
int	flag_disinf=0;
int	flag_rename=0;
int	flag_delete=0;
int	flag_append=0;	
int	flag_list=0;
int	flag_archives=0;
int	flag_passwd_archives=0;
int	flag_comp_exe=0;
int	flag_collect=0;
int	flag_ai=0;
int	flag_follow=0;
int	flag_old=0;

/* for radiobutton groups */
int	flag_scanmode=0;
int	flag_file_types=0;
int	flag_heur=0;
int	flag_macro=0;
int	flag_macro_action=0;

int have_version=0; 

GtkWidget	*main_window;
GtkWidget	*path_entry_box;
GtkWidget	*report_path_entry_box;
GtkWidget	*action_table;
GtkWidget	*disinfect_button;
GtkWidget	*rename_button;
GtkWidget	*delete_button;
GtkWidget	*macro_remove_table;
GtkWidget	*scan_1_button;
GtkWidget	*scan_2_button;
GtkWidget	*scan_3_button;
GtkWidget	*macro_1_button;
GtkWidget	*macro_2_button;
GtkWidget	*macro_3_button;
GtkWidget	*heur_1_button;
GtkWidget	*heur_2_button;
GtkWidget	*heur_3_button;
GtkWidget	*ai_check_button;
GtkWidget	*archives_button;
GtkWidget	*passwd_archives_button;

/*#################################### File Selection Widget wrappers  ##############################################*/

void do_file_selection(void)
{
	char *path=NULL;
	#ifdef USE_GTK_2_0
	char *filename=NULL;
	#endif

	path=file_selector(STR_SELECT_PATH_TO_SCAN, HIDE_FILE_OP_BUTTONS);

	debug(err_msg("%s() = %s",__FUNCTION__, (path)? path : "NULL"));

	if (path!=NULL) {
		#ifdef USE_GTK_2_0
		if ((filename=g_filename_to_utf8(path, -1,NULL,NULL,NULL))!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(path_entry_box),filename);
			filename=xfree(filename);
		}
		#else
		gtk_entry_set_text(GTK_ENTRY(path_entry_box), path);
		#endif
		path=xfree(path);
	}
	/*else  path_entry_box unchanged */
}

void do_report_file_selection(void)
{
	char *report_file=NULL;
	#ifdef USE_GTK_2_0
	char *filename=NULL;
	#endif

	report_file=file_selector(STR_SELECT_REPORT_FILE_PATH, HIDE_FILE_OP_BUTTONS);
	
	debug(err_msg("%s() = %s",__FUNCTION__, (report_file)? report_file : "NULL"));
	
	if (report_file!=NULL) {
		#ifdef USE_GTK_2_0
		if ((filename = g_filename_to_utf8(report_file,  -1,NULL,NULL,NULL))!=NULL) {
			gtk_entry_set_text(GTK_ENTRY(report_path_entry_box),filename);
			filename=xfree(filename);
		}
		#else
		gtk_entry_set_text(GTK_ENTRY(report_path_entry_box), report_file);
		#endif
		report_file=xfree(report_file);
	}
	/*else  report_path_entry_box unchanged */
}

/*############################### End of File Selection Widget wrappers #############################################*/


/*############################### Fprot-info, Virus-list, Update, Test & Reprot button functions ###################*/

#ifndef USE_GTK_2_0
void run(char *command_name)
{
	debug(err_msg("%s(): %s",__FUNCTION__, command_name));
	system(command_name);
}
#endif

void fprot_info(void)
{
	FILE	*fd;

#ifndef USE_GTK_2_0
	DBG_FUNCTION_NAME;
	
	
	if (have_version >= 313) {
		/*  f-prot version 3.13 and above */
		fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
	} else {
		/* This is for older versions of f-prot which don't have -verno */
		fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD2,"r");
	}
	textview_window_create(STR_INFO, 530, 430, 8, NULL, fd);
	xpclose_nostdin(fd);
#else
	int fd_int;
	char *argv_p[3];

	DBG_FUNCTION_NAME;	
	
	argv_p[0]=PATH_TO_F_PROT_BIN;
	argv_p[1]=INFO_CMD2;
	argv_p[2]=NULL;
	
	if (have_version >= 313) {
		/* f-prot version 3.13 and above */
		argv_p[1]=INFO_CMD;
	}
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_INFO, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
	#endif	
}

void fprot_virlist(void)
{
	FILE	*fd;

#ifndef USE_GTK_2_0
	DBG_FUNCTION_NAME;

	fd=xpopen(PATH_TO_F_PROT_BIN" "VIRLIST_CMD,"r");
	textview_window_create(STR_VIRUS_LIST, 530, 430, 8, NULL, fd);
	xpclose_nostdin(fd);
#else
	int fd_int;
	char *argv_p[3];
	
	DBG_FUNCTION_NAME;
	
	argv_p[0]=PATH_TO_F_PROT_BIN;
	argv_p[1]=VIRLIST_CMD;
	argv_p[2]=NULL;
	
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	/* don't test for NULL here this is done in textview_window_create */
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_VIRUS_LIST, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
	#endif
}

void fprot_update(void)
{
	DBG_FUNCTION_NAME;

	if ( have_version >= 410 ) {
		/* f-prot version 4.1.0 and above */
		#ifdef DEBUG
		msg("have_version %d", have_version);
		#endif
		#ifdef USE_GTK_2_0
		g_spawn_command_line_async (TERMINAL" "SU_SUDO" "UPDATE_CMD_VER_4_1, NULL);
		#else
		run(TERMINAL" "SU_SUDO" "UPDATE_CMD_VER_4_1" &");
		#endif
	} else {
		/* f-prot version prior to 4.1.0 */
		#ifdef DEBUG
		msg("have_version %d", have_version);
		#endif
		#ifdef USE_GTK_2_0
		g_spawn_command_line_async (TERMINAL" "SU_SUDO" "UPDATE_CMD, NULL);
		#else
		run(TERMINAL" "SU_SUDO" "UPDATE_CMD" &");
		#endif
	}
	/*
	* g_spawn_command_line_async:
	* @command_line: a command line
	* @error: return location for errors
	* 
	* A simple version of g_spawn_async() that parses a command line with
	* g_shell_parse_argv() and passes it to g_spawn_async(). Runs a
	* command line in the background. Unlike g_spawn_async(), the
	* %G_SPAWN_SEARCH_PATH flag is enabled, other flags are not. Note
	* that %G_SPAWN_SEARCH_PATH can have security implications, so
	* consider using g_spawn_async() directly if appropriate. Possible
	* errors are those from g_shell_parse_argv() and g_spawn_async().
	* 
	* The same concerns on Windows apply as for g_spawn_command_line_sync().
	*
	* Return value: %TRUE on success, %FALSE if error is set.
	*/
}

void fprot_test(void)
{
	FILE	*fd;
	
#ifndef USE_GTK_2_0
	DBG_FUNCTION_NAME;
	
	fd=xpopen(PATH_TO_F_PROT_BIN" "VIRTEST_CMD,"r");
	textview_window_create(STR_TEST, 530, 430, 8, NULL, fd);
	xpclose_nostdin(fd);
#else
	int fd_int;
	char *argv_p[3];

	DBG_FUNCTION_NAME;
	
	argv_p[0]=PATH_TO_F_PROT_BIN;
	argv_p[1]=VIRTEST_CMD;
	argv_p[2]=NULL;
	
	g_spawn_async_with_pipes (	NULL		/*working_directory*/,
								argv_p		/*argvp */,
								NULL		/*envp*/,
								0			/* flags*/,
								NULL		/* child_setup */,
								NULL		/* user_data */,
								NULL		/* child_pid */,
								NULL		/* standard_input*/,
								&fd_int 	/* standard_output */,
								NULL		/* standard_error */,
								NULL		/* error*/);	
	/* dont test for NULL here this is done in textview_window_create */
	fd = fdopen(fd_int, "r");
	textview_window_create(STR_TEST, 530, 430, 8, NULL, fd);
	xfclose_nostdin(fd);
	#endif	
}

void view_report(void)
{
	char *report=NULL;

	DBG_FUNCTION_NAME;

	#ifdef USE_GTK_2_0
	report=g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1,NULL,NULL,NULL);
	textedit_window_create(report,530,450,8,report);
	report=xfree(report);
	#else
	report =(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	textedit_window_create(report,530,450,8,report);
	#endif
}

/*######################### END of Fprot-info, Virus-list, Update, Test & Report button functions ###################*/

/*############################# START of Scan button function  ######################################################*/

char * add_to_command(char * command, char *option)
{
	int size;
	size=xstrlen(option)+xstrlen(command)+1;
	command=(char*)xrealloc(command,size);
	xstrncat(command,option,size);
	debug(err_msg("%s(): %s",__FUNCTION__, option));
	return command;
}

#define FORMAT_STRING		"%s %s \"%s\""

void do_scan_f(void)
{
	int size;
	char * scan_path=NULL;
	char * report_path=NULL;
	/*char * fmt="%s %s \"%s\"";*/
	char * command=NULL;

	DBG_FUNCTION_NAME;

	#ifdef USE_GTK_2_0
	scan_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(path_entry_box)), -1, NULL, NULL, NULL);
	#else
	scan_path =(char *) gtk_entry_get_text(GTK_ENTRY(path_entry_box));
	#endif

	size = xstrlen(TERMINAL)+ xstrlen(PATH_TO_F_PROT_BIN) + xstrlen(scan_path)+xstrlen(FORMAT_STRING)+1;
	command=(char *)xmalloc(size);

	/* We always "" quote the path to scan to avoid troubles with filenames with spaces
		which are split by the shell and mess up our command line args */

	xsnprintf(command, size, FORMAT_STRING, TERMINAL, PATH_TO_F_PROT_BIN, scan_path);

	/* I prefer to create always a report file so no flag checking here.*/
	/* BTW we always quote filenames and paths */
	command=add_to_command(command, " -report=");
	command=add_to_command(command, "\"");

	#ifdef USE_GTK_2_0
	report_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);
	#else
	report_path=(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	#endif

	command=add_to_command(command, report_path);
	command=add_to_command(command, "\"");

	/* Let's parse the options*/
	if (flag_scanmode > 0 ) { /* not in report only mode */
		if (IS_SET(flag_disinf))
			command=add_to_command(command," -disinf");
		if (IS_SET(flag_rename))
			command=add_to_command(command," -rename");
		if (IS_SET(flag_delete))
			command=add_to_command(command," -delete");
		
		/* flag_scanmode == 1 is the default: Interactive */
		if (flag_scanmode==2) /* Automatic */
			command=add_to_command(command," -auto");
	}

	switch (flag_file_types) {
		case 0:
			/* "Scan all files" this is the default */
			command=add_to_command(command," -dumb");
			break;
		case 1:
			/* "Standard file extensions" */
			command=add_to_command(command," -ext");
			break;
		case 2:
			/* "Attempt to identify files" */
			command=add_to_command(command," -type");
			break;
		#ifdef DEBUG
		default:
			err_msg("Bad value: flag_file_types");
			exit(EXIT_FAILURE);
		#endif
	}

	if (IS_SET(flag_archives)) {
		/*	f-prot 4.5.3 adds a new option: 
			-archive=n [default is 5].
			Scan inside supported archives n levels deep,
			the supported range is between 1 and 99,
			the default level is 5.
			For the moment we stick with the default.
			TODO
			maybe I'll add a spinbutton in the future.
		*/
		command=add_to_command(command," -archive");
		if (have_version >= 440) { 
			/* f-prot version 4.4.1 and above */
			/* Attempts to identify infections within password protected archives. The option -server implies -archive.*/
			if (IS_SET(flag_passwd_archives))
				command=add_to_command(command," -server");
			else
				command=add_to_command(command," -noserver");	
		}
	} else {
		if (have_version >= 440) { 
			/* f-prot version 4.4.1 and above */
			/*Does not scan inside archives. The option -noarchive implies -noserver.*/
			command=add_to_command(command," -noarchive");
			command=add_to_command(command," -noserver");	
		}
	}
	 /* Unpack compressed executables. There is no corresponding -nopacked option. 
	 This option is provided for legacy reasons. */
	if (IS_SET(flag_comp_exe))
		command=add_to_command(command," -packed");
	if (IS_SET(flag_nosub))
		command=add_to_command(command," -nosub");
	if (IS_SET(flag_collect))
		command=add_to_command(command," -collect");
	if (IS_SET(flag_list))
		command=add_to_command(command," -list");
	if (IS_SET(flag_ai))
		command=add_to_command(command," -ai");
	if (IS_SET(flag_old))
		command=add_to_command(command," -old");
	/* "Follow symbolic links" is not displayed in f-prot switches but works correctly	*/
	if (IS_SET(flag_follow))
		command=add_to_command(command," -follow");
	if (IS_SET(flag_append))
		command=add_to_command(command," -append");

	switch (flag_heur) {
		case 1:
			command=add_to_command(command," -noheur");
			break;
		case 2:
			command=add_to_command(command," -onlyheur");
			break;
		#ifdef DEBUG
		case 0:
			/* flag_heur == 0 is default */
			break;
		default:
			err_msg("Bad value: flag_heur");
			exit(EXIT_FAILURE);
		#endif
	}
	
	switch (flag_macro) {
		case 1:
			command=add_to_command(command," -nomacro");
			break;
		case 2:
			command=add_to_command(command," -onlymacro");
			break;
		#ifdef DEBUG
		case 0:
			/* flag_macro == 0 is default */
			break;
		default:
			err_msg("Bad value: flag_macro");
			exit(EXIT_FAILURE);
		#endif
	}
		
	if (flag_scanmode > 0 && flag_macro > 0 ) {
		switch (flag_macro_action) {
			case 0:
				/*"Remove infected macros from docs"*/
				command=add_to_command(command," -saferemove");
				break;
			case 1:
				/*"Remove all macros from infected docs"*/
				command=add_to_command(command," -removenew");
				break;
			case 2:
				/*"Remove all macros from all docs"*/
				command=add_to_command(command," -removeall");
				break;
			#ifdef DEBUG
			default:
				err_msg("Bad value: flag_macro_action");
				exit(EXIT_FAILURE);
			#endif
		}
			
	}
	/*  -wrap switch 	is not supported, if you want to edit the logs use the text editor */
	/*  -nobreak    	is not supported (Do not abort scan if ESC is pressed) */
	/*  -page       	is not supported (Pause after each page.)*/
	/*  -silent     	is not supported (Do not generate any screen output.)*/

	msg("executing scan on: %s", scan_path);
	#ifdef USE_GTK_2_0
	g_spawn_command_line_async (command, NULL);
	scan_path=xfree(scan_path);
	report_path=xfree(report_path);
	#else
	command=add_to_command(command, " &");
	run(command);
	#endif
	command=xfree(command);
}

void scan_f(void)
{
	char *report_path=NULL;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;

	/* Check if at least a flag for actions is set, if not make sure that scanmode is set to report only  and return*/
	if ((IS_NOT_SET(flag_disinf) && IS_NOT_SET(flag_rename) && IS_NOT_SET(flag_delete)) && flag_scanmode > 0 ) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button),TRUE);
		SET2(flag_scanmode,0);
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_NO_ACTION_DOING_REPORT_ONLY);
		return;
	}
	
	#ifdef USE_GTK_2_0
	report_path =g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(report_path_entry_box)), -1, NULL,NULL,NULL);
	#else
	report_path=(char *) gtk_entry_get_text(GTK_ENTRY(report_path_entry_box));
	#endif

	if (lstat(report_path,&stat_p)==0) {/* File exists */
		if (!S_ISREG(stat_p.st_mode)) { /* is not a regular file */
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_REPORT_FILE_IS_NOT_A_REGULAR_FILE);
			return;
		} else { /* is a regular file */
			if (IS_NOT_SET(flag_append)) { /* Don't append */
				if (dialog_window(STR_WARNING, TWO_BUTTONS, NULL, 2, STR_REPORT_FILE_EXISTS_OVERWRITE)==1) {
					/* Don't overwrite */
					return;
				}
				/*else overwrite existing report */
			}
			/*else append to existing report */
		}
	}
	#ifdef USE_GTK_2_0
	report_path=xfree(report_path);
	#endif
	/*else file don't exist so write it now */
	do_scan_f();
}

/*############################## End of Scan button function  ######################################################*/

void remove_lock(void)
{
	DBG_FUNCTION_NAME;

	if (remove(LOCK_FILE)==-1) {
		err_msg("failed to remove lock file: %m");
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_FAILED_TO_REMOVE_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	msg("removed lock file");
}

void about_f(void)
{
	FILE	*fd;

	fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
	about_window_create(VERSION, 620, 430, 8, VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
						fd,NOT_FIRST_TIME);
	xfclose_nostdin(fd);
}

void create_file(char * filename)
{
	int pfd;

	if ((pfd=open(filename ,O_RDWR|O_CREAT, 0600))!=-1)
			close(pfd);
	#ifdef DEBUG
	else
		err_msg("cannot open file: %s: %m", filename); /* This error is ignored */
	#endif
}

void root_warn(void)
{
	int ret;
	struct stat stat_p;

	DBG_FUNCTION_NAME;

	msg("you are root!");

	if (lstat(NO_ROOT_WARN_FILE,&stat_p)!=0) {
		ret=dialog_window(VERSION" - "STR_WARNING, TWO_BUTTONS_ONE_CHECK, STR_DONT_SHOW_IT_AGAIN,
															2, STR_YOU_ARE_RUNNING_XFPROT_AS_ROOT_CONTINUE);
		switch (ret) {
			case 4: /* No + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
				/* FALLTROUGH */
			case 1: /* No */
				remove_lock();
				msg("...exiting");
				exit(EXIT_SUCCESS);
			case 0: /* Ok */
				break;
			case 3: /* Ok + checkbutton */
				create_file(NO_ROOT_WARN_FILE);
				break;
			/*esac */
		}
	}
	#ifdef DEBUG
	else
		err_msg("NO_ROOT_WARN_FILE");
	#endif
}

void do_lock_file(void)
{
	int pfd;
	int dialog_retval=-1;
	struct	stat stat_p;

	DBG_FUNCTION_NAME;
	
	/* Check if LOCK FILE exists and remove if stale */
	
	/* File exists */
	if (lstat(LOCK_FILE,&stat_p)==0) {
		err_msg("lock file found");

		dialog_retval = dialog_window(VERSION" - "STR_WARNING, TWO_BUTTONS, NULL, 2, STR_XFPROT_HAS_DETECTED_A_LOCK_FILE);
		if (dialog_retval==1) {
			msg("...exiting");
			exit(EXIT_SUCCESS);
		}
		remove_lock();
	}
	/* cannot open */
	if ((pfd=open(LOCK_FILE,O_RDWR|O_CREAT|O_EXCL,0640))<0) {
		err_msg("cannot open lock file: %m");
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_OPEN_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* cannot lock */
	if (lockf(pfd,F_TLOCK,0)<0) {
		err_msg("lockf(): %m");
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_LOCK_FILE_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* only first instance continues */
	close(pfd);
}

void set_dir_properties_and_cd_in(uid_t uid, gid_t gid)
{
	DBG_FUNCTION_NAME;

	msg("setting permissions of private dir to 0700");
	if (xchmod(XFPROT_PRIV_DIR,0700, RETURN_ON_FAILURE)==-1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SET_PERMISSIONS_OF_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}

	msg("setting owner of private dir");
	if (xchown(XFPROT_PRIV_DIR, uid, gid, RETURN_ON_FAILURE)==-1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_SET_OWNER_OF_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}

	msg("chdir to private dir");
	if (xchdir(XFPROT_PRIV_DIR, RETURN_ON_FAILURE)==-1) {
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_CHANGE_DIR_TO_PRIVATE_DIR_ABORTING);
		exit(EXIT_FAILURE);
	}
}

void check_for_bin(void)
{
	DBG_FUNCTION_NAME;

	/* access  may  not  work  correctly  on NFS file systems */
	if (access(PATH_TO_F_PROT_BIN , X_OK)<0) {
		err_msg("access(): %s: %m",PATH_TO_F_PROT_BIN);
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, PATH_TO_F_PROT_BIN"\n"STR_FILE_NOT_FOUND_ABORTING);
		exit(EXIT_FAILURE);
	}
}

void toggle1_f(void)
{
	DBG_FUNCTION_NAME;

	if (flag_scanmode > 0 && flag_macro < 2 ) {
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(action_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disinfect_button),	TRUE);
		SET(flag_disinf);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rename_button),	FALSE);
		RESET(flag_rename);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (delete_button),	FALSE);
		RESET(flag_delete);
	} else {
		gtk_widget_hide(action_table);
		debug(err_msg("hiding widget action_table"));
	}
}

void toggle2_f(void)
{
	DBG_FUNCTION_NAME;

	if ( flag_scanmode > 0 && flag_macro != 1 ) {
		/* It's better to reset the flags to default values
			here to avoid leftovers from previous toggles */
		gtk_widget_show(macro_remove_table);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (macro_1_button), TRUE);
		SET2(flag_macro_action,0);
	} else {
		gtk_widget_hide(macro_remove_table);
		debug(err_msg("hiding widget macro_remove_table"));
	}
}

void flag_scanmode_callback( GtkWidget *widget, gpointer callback_data )
{
	assert(widget!=NULL);

	SET2(flag_scanmode,atoi(callback_data));
	toggle1_f();
	toggle2_f();
}

void flag_file_types_callback(GtkWidget *widget, gpointer callback_data )
{
	assert(widget!=NULL);

	SET2(flag_file_types,atoi(callback_data));
}

void flag_macro_callback(GtkWidget *widget, gpointer callback_data )
{
	assert(widget!=NULL);

	SET2(flag_macro,atoi(callback_data));
	toggle2_f();
	toggle1_f();
}

void flag_macro_action_callback(GtkWidget *widget, gpointer callback_data )
{
	assert(widget!=NULL);

	SET2(flag_macro_action,atoi(callback_data));
}

void flag_heur_callback(GtkWidget *widget, gpointer callback_data )
{
	assert(widget!=NULL);

	SET2(flag_heur,atoi(callback_data));
	/* Disable neural-network virus detection. The -ai option should not be used with the -noheur option.*/
	if (flag_heur==1)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button),FALSE);
}

void flag_ai_callback(GtkWidget *widget, int *callback_data )
{
	assert(widget!=NULL);

	if (IS_SET(*callback_data)) {
		RESET(*callback_data);
	} else {
		/* Enable heuristics.The -ai option (neural-network virus detection) should not be used with the -noheur option.*/
		if (flag_heur==1) {
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (heur_1_button), TRUE);
			SET2(flag_heur,0);
		}
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ai_check_button),TRUE);
		SET(*callback_data);	
	}
}

void flag_archives_callback(GtkWidget *widget, int *callback_data )
{
	assert(widget!=NULL);

	if (IS_SET(*callback_data)) { 
		RESET(*callback_data);
		if (have_version >= 440 ) { /* f-prot version 4.4.1 and above */
			/* check for flag_protected_archives and disable if set */
			if (IS_SET(flag_passwd_archives)) {
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (passwd_archives_button),FALSE);
				SET2(flag_passwd_archives,0);
			}
		}
	} else {
		SET(*callback_data);	
	}
}

void flag_passwd_archives_callback(GtkWidget *widget, int *callback_data )
{
	assert(widget!=NULL);
	
	if (have_version >= 440) { /* f-prot version 4.4.1 and above */
		if (IS_SET(*callback_data)) {
			RESET(*callback_data);
		} else {
			/* check for flag_archives and enable if unset */
			if (IS_NOT_SET(flag_archives)) {
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (archives_button),TRUE);
				SET2(flag_archives,1);
			}
			SET(*callback_data);	
		}
	}
}

void flag_callback(GtkWidget *widget, int *callback_data )
{
	assert(widget!=NULL);

	if (IS_SET(*callback_data))
		RESET(*callback_data);
	else
		SET(*callback_data);	
}

void signal_handler(int sig)
{
	switch (sig) {
		case SIGINT:
				msg("SIGINT signal catched");
				main_window_close(main_window);
				break;
		case SIGTERM:
				msg("SIGTERM signal catched");
				main_window_close(main_window);
				break;
	}
}

#ifdef USE_GTK_2_0
#ifdef USE_GTK_2_4

static GtkActionEntry xfprot_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{ "FileMenu",	NULL,		"_"STR_FILE,		NULL,				NULL,		NULL},
	{ "ToolsMenu",	NULL,		"_"STR_TOOLS,		NULL,				NULL,		NULL},
	{ "HelpMenu",	NULL,		"_"STR_HELP,		NULL,				NULL,		NULL},
	{ "Scan",		NULL,		STR_SCAN, 			NULL,				NULL,		G_CALLBACK (scan_f) },
	{ "Update",		NULL,		STR_UPDATE,			NULL,				NULL,		G_CALLBACK (fprot_update) },
	{ "Report",		NULL,		STR_REPORT,			NULL,				NULL,		G_CALLBACK (view_report) },
	{ "Quit",		NULL,		"_"STR_QUIT,		SHC_FILE_QUIT,		NULL,		G_CALLBACK (main_window_close) },
	{ "Test",		NULL,		STR_TEST,			NULL,				NULL,		G_CALLBACK (fprot_test) },
	{ "Info",		NULL,		STR_INFO,			NULL,				NULL,		G_CALLBACK (fprot_info) },
	{ "VirusList",	NULL,		STR_VIRUS_LIST,		NULL,				NULL,		G_CALLBACK (fprot_virlist) },
	{ "About",		NULL,		STR_ABOUT,			NULL,				NULL,		G_CALLBACK (about_f) },
};

static const gchar *xfprot_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='FileMenu'>"
"      <menuitem action='Scan'/>"
"      <menuitem action='Update'/>"
"      <menuitem action='Report'/>"
"      <separator/>"
"      <menuitem action='Quit'/>"
"    </menu>"
"    <menu action='ToolsMenu'>"
"      <menuitem action='Test'/>"
"      <menuitem action='Info'/>"
"      <menuitem action='VirusList'/>"
"    </menu>"
"    <menu action='HelpMenu'>"
"      <menuitem action='About'/>"
"    </menu>"
"  </menubar>"
"</ui>";

struct MenuEntry xfprot_menu_entries[] = {
{ xfprot_entries,		NULL,						NULL,			G_N_ELEMENTS (xfprot_entries),				0, NULL},
{ NULL,					NULL,						NULL,			0,											0, NULL},
/* Set last to NULL */ 
};
#else
static GtkItemFactoryEntry xfprot_menu_items[] = {
	{ "/_"STR_FILE,						NULL,				NULL,				0,	"<Branch>",		NULL},
	{ "/"STR_FILE"/"STR_SCAN,			NULL,				scan_f,				1,	NULL,			NULL},
	{ "/"STR_FILE"/"STR_UPDATE,			NULL,				fprot_update,		2,	NULL,			NULL},
	{ "/"STR_FILE"/"STR_REPORT,			NULL,				view_report,		3,	NULL,			NULL},
	{ "/"STR_FILE"/sep1",				NULL,				NULL,				0, "<Separator>",	NULL},
	{ "/_"STR_QUIT,						SHC_FILE_QUIT,		main_window_close,	4,	NULL,			NULL},
	{ "/_"STR_TOOLS,					NULL,				NULL,				0,	"<Branch>",		NULL},
	{ "/"STR_TOOLS"/"STR_TEST,			NULL,				fprot_test,			5,	NULL,			NULL},
	{ "/"STR_TOOLS"/"STR_INFO,			NULL,				fprot_info,			6,	NULL,			NULL},
	{ "/"STR_TOOLS"/"STR_VIRUS_LIST,	NULL,				fprot_virlist,		7,	NULL,			NULL},
	{ "/_"STR_HELP,						NULL,				NULL,				0,	"<LastBranch>",	NULL},
	{ "/"STR_HELP"/"STR_ABOUT,			NULL,				about_f,			9,	NULL,			NULL},
	{ NULL,								NULL,				NULL,				0,	NULL,			NULL},
	 /* Set last item to NULL */
};
#endif
#else
static GtkItemFactoryEntry xfprot_menu_items[] = {
	{ "/_"STR_FILE,						NULL,				NULL,				0,	"<Branch>"		},
	{ "/"STR_FILE"/"STR_SCAN,			NULL,				scan_f,				1,	NULL			},
	{ "/"STR_FILE"/"STR_UPDATE,			NULL,				fprot_update,		2,	NULL			},
	{ "/"STR_FILE"/"STR_REPORT,			NULL,				view_report,		3,	NULL			},
	{ "/"STR_FILE"/sep1",				NULL,				NULL,				0, "<Separator>"	},
	{ "/"STR_FILE"/_"STR_QUIT,			SHC_FILE_QUIT,		main_window_close,	4,	NULL			},
	{ "/_"STR_TOOLS,					NULL,				NULL,				0,	"<Branch>"		},
	{ "/"STR_TOOLS"/"STR_TEST,			NULL,				fprot_test,			5,	NULL			},
	{ "/"STR_TOOLS"/"STR_INFO,			NULL,				fprot_info,			6,	NULL			},
	{ "/"STR_TOOLS"/"STR_VIRUS_LIST,	NULL,				fprot_virlist,		7,	NULL			},
	{ "/_"STR_HELP,						NULL,				NULL,				0,	"<LastBranch>"	},
	{ "/"STR_HELP"/"STR_ABOUT,			NULL,				about_f,			9,	NULL			},
	{ NULL,								NULL,				NULL,				0,	NULL			},
	 /* Set last item to NULL */
};
#endif

/* This callback quits the program */
void main_window_close(GtkWidget *widget)
{
	DBG_FUNCTION_NAME;

	gtk_main_quit ();
#ifdef USE_GTK_2_0	
	gtk_widget_destroy(main_window);
#endif	
	remove_lock();
	msg("...exiting");
	/* To shut a compiler warning */
	widget=NULL;
}


int main(int argc, char **argv)
{
	GtkWidget		*label;
	GtkWidget		*top_table;
	GtkWidget		*mode_table;
	GtkWidget		*macro_table;
	GtkWidget		*file_types_table;
	GtkWidget		*report_table;
	GtkWidget		*option_table;
	GtkWidget		*frame1;
	GtkWidget		*frame2;
	GtkWidget		*frame3;
	GtkWidget		*frame4;
	GtkWidget		*frame5;
	GtkWidget		*frame6;
	GtkWidget		*frame7;
	GtkWidget		*table1;
	GtkWidget		*button1;
	GtkWidget		*button2;
	GtkWidget		*button3;
	GtkWidget		*button4;
	GtkWidget		*button5;
	GtkWidget		*button6;
	GtkWidget		*button7;
#ifdef USE_GTK_2_0
	GtkWidget		*button8;
#endif
	GtkWidget		*radio_button1;
	GtkWidget		*radio_button2;
	GtkWidget		*radio_button3;
	GtkWidget		*radio_button4;
	GtkWidget		*radio_button5;
	GtkWidget		*radio_button6;
	GtkWidget		*check_button1;
	GtkWidget		*check_button2;
	GtkWidget		*check_button3;
	GtkWidget		*check_button4;
	GtkWidget		*check_button5;
	GtkWidget		*check_button6;
	GtkWidget		*check_button7;
	GtkWidget		*menubar;
	GtkTooltips		*action_tooltip;
	GtkTooltips		*onlyheur_tooltip;
	GSList			*scan_mode_group=NULL;
	GSList			*scan_macro_group=NULL;
	GSList			*file_types_group=NULL;
	GSList			*remove_macro_group=NULL;
	GSList			*heuristics_group=NULL;

	int	*ptr_flag_nosub;
	int	*ptr_flag_disinf;
	int	*ptr_flag_rename;
	int	*ptr_flag_delete;
	int	*ptr_flag_append;	
	int	*ptr_flag_list;
	int	*ptr_flag_archives;
	int *ptr_flag_passwd_archives;
	int	*ptr_flag_comp_exe;
	int	*ptr_flag_collect;
	int	*ptr_flag_ai;
	int	*ptr_flag_follow;
	int	*ptr_flag_old;
	
	char tmp[64];
	char *path=NULL;
	char *report_file_path=NULL;
	FILE	*fd;


	struct	passwd *pw;
	struct	stat stat_p;

	/* print disclaimer to terminal */
	xputs("\n"VERSION"\n\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY"\n\n");
	msg("starting...");

	signal(SIGHUP,SIG_IGN);  			/* ignore SIGHUP */
	signal(SIGTERM,signal_handler);		/* catch SIGTERM */
	signal(SIGINT,signal_handler);		/* catch SIGINT  */

	/* Initializes internationalization support for GTK+.
	 gtk_init() automatically does this, so there is 
	 typically no point in calling this function. */
	/*gtk_set_locale();*/
	gtk_init (&argc, &argv);

	/* Force the output to be unbuffered when debugging.*/
	debug(setbuf(stdout, (char *) 0));

	/* Command line args. No dialog windows on errors as this is console stuff */
	if ( argc == 2) {
		if (strcmp(argv[1],"-v")==0) {
			msg(VERSION);
			exit(EXIT_SUCCESS);
		}
		err_msg("unknown command line switch: %s",argv[1]);
		exit(EXIT_FAILURE);
	}
	if (argc > 2) {
		err_msg("too much args on command line");
		exit(EXIT_FAILURE);
	}

	#ifdef NO_CORE_DUMPS
	/* make the process not dumpable under Linux 2.4, seems to work also for 2.6.0-test3 */
	prctl(PR_SET_DUMPABLE,0);
	msg("process is%sdumpable",(prctl(PR_GET_DUMPABLE)==0)?" not ":" ");
	#endif

	/* Try to determine the home dir of the user in
		passwd struct to avoid use of getenv("HOME").
		Create a private xfprot dir in user's home dir
		if it doesn't already exist.
		Set explicitly ownership and permissions of xfprot's dir.
		If something goes wrong we issue an error and exit.
		BTW: the user is forced to write the report always in
		xfprot's  private dir.
	*/

	/* Who am I ? */
	if ((pw=getpwuid(getuid()))==NULL) {
		debug(err_msg("getpwuid(getuid()): %m"));
		dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_GET_PWENT_ABORTING);
		exit(EXIT_FAILURE);
	}
	/* chdir to user's home dir */
	if ((xchdir(pw->pw_dir, RETURN_ON_FAILURE))==EXIT_ERROR) {
		dialog_window(VERSION" - "STR_ALERT, ONE_BUTTON, NULL, 1, STR_FAILED_TO_CD_TO_HOMEDIR_TRYING_ROOTDIR);
		if (xchdir("/",RETURN_ON_FAILURE)==-1) {
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_FAILED_TO_CD_TO_ROOTDIR_ABORTING);
			exit(EXIT_FAILURE);
		}
	}
	/* Check for xfprot's private dir */
	if (lstat(XFPROT_PRIV_DIR,&stat_p)==0) {
		msg("private dir found");
		if (S_ISLNK(stat_p.st_mode)) {
			/* We don't want to be redirected by a link */
			debug(err_msg("private dir is a link"));
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_PRIVATE_DIR_IS_A_LINK_ABORTING);
			exit(EXIT_FAILURE);
		}
		#ifdef DEBUG
		if (S_ISDIR(stat_p.st_mode)) {
			/* OK let's go on */
			err_msg("good...private dir is not a link");
		}
		#endif
	} else {
		err_msg("private dir not found");
		if (xmkdir(XFPROT_PRIV_DIR, 0700, RETURN_ON_FAILURE)==-1) {
			dialog_window(VERSION" - "STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_CREATE_PRIVATE_DIR_ABORTING);
			exit(EXIT_FAILURE);
		}
	}

	/* Explicitly set private dir's permissions and owner and chdir into it */
	set_dir_properties_and_cd_in( pw->pw_uid, pw->pw_gid);
	/* check for f-prot*/
	check_for_bin();
	/* lock file stuff */
	do_lock_file();
	/* display a warning if we are root */
	if (pw->pw_uid==0 )
		root_warn();

	/* Detect f-prot version 4.4.x for -server/-noserver switch and detect -verno switch */
	fd=xpopen(PATH_TO_F_PROT_BIN" "INFO_CMD,"r");
	while (fgets(tmp, sizeof(tmp), fd)!=NULL) {
		/* Detect -verno f-prot >=3.13 */
		if (strstr(tmp,"F-PROT ANTIVIRUS")!=NULL)
			SET2(have_version, 313);
			/* f-prot version 3.13 */			
		/* Detect version */
		if (strstr(tmp,"Program version: 4.1")!=NULL)
			SET2(have_version, 410);
			/* f-prot version 4.1.x */		
		if (strstr(tmp,"Program version: 4.4")!=NULL)
			SET2(have_version, 440);
			/* f-prot version 4.4.x */
		if (strstr(tmp,"Program version: 4.5")!=NULL)
			SET2(have_version, 450);
			/* f-prot version 4.5.x */
		if (strstr(tmp,"Program version: 4.6")!=NULL)
			SET2(have_version, 460);
			/* f-prot version 4.6.x */
	}
	debug(err_msg(" f-prot version %d", have_version));
	xpclose_nostdin(fd);

	/* Initialize our flags */	
	ptr_flag_nosub	=	&flag_nosub;
	ptr_flag_disinf	=	&flag_disinf;
	ptr_flag_rename	=	&flag_rename;
	ptr_flag_delete	=	&flag_delete;
	ptr_flag_append	=	&flag_append;	
	ptr_flag_list	=	&flag_list;
	ptr_flag_archives =	&flag_archives;
	ptr_flag_passwd_archives = &flag_passwd_archives;
	ptr_flag_comp_exe =	&flag_comp_exe;
	ptr_flag_collect =	&flag_collect;
	ptr_flag_ai		=	&flag_ai;
	ptr_flag_follow =	&flag_follow;
	ptr_flag_old	=	&flag_old;
	
	main_window = window_create(640,  450, 0, VERSION,GTK_SIGNAL_FUNC(main_window_close), NULL, MODAL);

	table1 = table_create(10, 21, main_window, 2 ,  NOT_HOMOGENEUS);
	#ifdef USE_GTK_2_4
	menubar = main_menu_create(GTK_WIDGET (main_window), xfprot_menu_entries, xfprot_ui_info, NULL);
	#else		
	menubar = main_menu_create(main_window, (GtkItemFactoryEntry *) &xfprot_menu_items, NULL);
	#endif
	gtk_table_attach_defaults (GTK_TABLE(table1),menubar, 0, 10, 0, 1);
	
	frame1 = frame_create_on_table(STR_PATH_TO_SCAN, table1, 0, 10, 1,4);
	top_table = table_create(20, 2, frame1, 2 , HOMOGENEUS);
	path=xgetcwd(path);

	debug(err_msg(path));

	path_entry_box = text_box_create_on_table(path, NOT_EDITABLE, top_table, 0, 19, 0, 1);

	button1 = button_create_on_table(":",GTK_SIGNAL_FUNC(do_file_selection), NULL, top_table, 19, 20, 0,1);

	check_button1 = check_create_on_table(STR_NOSUB, GTK_SIGNAL_FUNC(flag_callback), ptr_flag_nosub, 
										INACTIVE, top_table, 0, 10, 1, 2);

	frame2 = frame_create_on_table(STR_SCAN_MODE, table1, 0, 5, 4, 8);
	mode_table = table_create(1, 3, frame2, 2 , HOMOGENEUS);

	/* Warning we can't use the ACTIVE flag here 'cause the callback function acts on widgets
		we haven't created yet, so we do things manually later by callin toggle1_f() */
	scan_1_button = radio_create_on_table(NULL, STR_REPORT_ONLY, GTK_SIGNAL_FUNC(flag_scanmode_callback), "0",
													INACTIVE, mode_table, 0, 1, 0, 1);

	/* Maybe this is not needed as first radio button is set active by default, but to be sure.....*/
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (scan_1_button),TRUE);

	scan_mode_group = get_radio_button_group (GTK_RADIO_BUTTON (scan_1_button));
	scan_2_button = radio_create_on_table(scan_1_button,STR_INTERACTIVE,GTK_SIGNAL_FUNC(flag_scanmode_callback),"1",
													INACTIVE,mode_table, 0, 1, 1,2);
	scan_3_button = radio_create_on_table(scan_2_button, STR_AUTOMATIC,GTK_SIGNAL_FUNC(flag_scanmode_callback),"2",
													INACTIVE,mode_table, 0, 1, 2,3);

	frame3 = frame_create_on_table( STR_SCAN_FOR_MACRO_VIRUS, table1, 5, 10, 4, 8);
	macro_table = table_create(1, 3, frame3, 2 ,  HOMOGENEUS);

	/* Warning we can't use the ACTIVE flag here because the callback function acts on widgets
		we haven't created yet, so we do things manually */
	radio_button1 = radio_create_on_table(NULL, STR_YES, GTK_SIGNAL_FUNC(flag_macro_callback),
										"0", INACTIVE, macro_table, 0, 1, 0, 1);

	/* Maybe this is not needed as first radio button is set active by default, but to be sure.....*/
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button1),TRUE);

	scan_macro_group = get_radio_button_group (GTK_RADIO_BUTTON (radio_button1));
	radio_button2 = radio_create_on_table(radio_button1,STR_NO,GTK_SIGNAL_FUNC(flag_macro_callback),
										"1", INACTIVE, macro_table, 0, 1, 1, 2);
	radio_button3 = radio_create_on_table(radio_button2,STR_ONLY,GTK_SIGNAL_FUNC(flag_macro_callback),
										"2", INACTIVE, macro_table, 0, 1, 2, 3);
	
	frame4 = frame_create_on_table( STR_ACTIONS_TO_TRY_ON_INFECTED_FILES, table1, 0, 5, 8, 12);
	action_table = table_create(1, 3, frame4, 2 ,  HOMOGENEUS);
	action_tooltip=gtk_tooltips_new();

	#ifndef USE_GTK_2_0
	/* Warning: Gtk+ 1.2.10 docs for gtk_tooltips_set_delay say:
	 * delay: an integer value representing seconds !!!
	 * but in gtktooltips.c  we read #define DEFAULT_DELAY 500   (Default delay in ms)
	 * So this seems to works for me only using seconds*1000
	 */
	gtk_tooltips_set_delay (action_tooltip,2000); /* 2 seconds */
	#endif

	disinfect_button =check_create_on_table(	STR_DISINFECT_WHENEVER_POSSIBLE,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_disinf,
												ACTIVE,action_table, 0, 1, 0, 1);

	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),disinfect_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);

	rename_button =		check_create_on_table(	STR_RENAME_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_rename,
												INACTIVE, action_table, 0, 1, 1, 2);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),rename_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);
	delete_button =		check_create_on_table(	STR_DELETE_INFECTED_FILES,
												GTK_SIGNAL_FUNC(flag_callback), ptr_flag_delete,
												INACTIVE, action_table, 0, 1, 2, 3);
	gtk_tooltips_set_tip (GTK_TOOLTIPS (action_tooltip),delete_button,
				STR_COMBINATIONS_OF_SWITCHES,NULL);
	gtk_tooltips_enable (action_tooltip);

	frame5 = frame_create_on_table(STR_MACRO_VIRUS_REMOVAL_OPTIONS, table1, 5, 10, 8, 12);
	macro_remove_table = table_create(1, 3, frame5, 2 ,  HOMOGENEUS);
	macro_1_button = radio_create_on_table(	NULL, STR_REMOVE_INFECTED_MACROS_FROM_DOCS,
											GTK_SIGNAL_FUNC(flag_macro_action_callback), "0",
											ACTIVE, macro_remove_table, 0, 1, 0, 1);
	remove_macro_group = get_radio_button_group (GTK_RADIO_BUTTON (macro_1_button));
	macro_2_button = radio_create_on_table(	macro_1_button,STR_REMOVE_ALL_MACROS_FROM_INFECTED_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "1",
													INACTIVE, macro_remove_table, 0, 1, 1, 2);
	macro_3_button = radio_create_on_table(	macro_2_button,STR_REMOVE_ALL_MACROS_FROM_ALL_DOCS,
													GTK_SIGNAL_FUNC(flag_macro_action_callback), "2",
													INACTIVE, macro_remove_table, 0, 1, 2, 3);

	frame6 = frame_create_on_table(STR_FILE_TYPES, table1, 0, 5, 12, 16);
	file_types_table = table_create(1, 3, frame6, 2 ,  HOMOGENEUS);

	radio_button4 = radio_create_on_table(NULL,	STR_SCAN_ALL_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"0",
										ACTIVE,file_types_table, 0, 1, 0, 1);
	file_types_group = get_radio_button_group (GTK_RADIO_BUTTON (radio_button4));
	radio_button5 = radio_create_on_table(radio_button4,STR_STANDARD_FILE_EXTENSIONS,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"1",
										INACTIVE,file_types_table, 0, 1, 1, 2);
	radio_button6 = radio_create_on_table(radio_button5,STR_ATTEMPT_TO_IDENTIFY_FILES,
										GTK_SIGNAL_FUNC(flag_file_types_callback),"2",
										INACTIVE,file_types_table, 0, 1, 2, 3);
										
	frame6 = frame_create_on_table(STR_REPORT_FILE, table1, 0, 5, 16, 20);
	report_table = table_create(8, 3, frame6, 2 , HOMOGENEUS);


	report_file_path=xconcat_path_file(path, DEF_REPORT_FILE);

	report_path_entry_box = text_box_create_on_table( report_file_path , NOT_EDITABLE, report_table, 0, 7, 0, 1);
	button2 = button_create_on_table(":" ,GTK_SIGNAL_FUNC(do_report_file_selection)	, NULL, report_table, 7, 8, 0, 1);

	report_file_path=xfree(report_file_path);
	path=xfree(path);

	check_button2 = check_create_on_table(STR_APPEND_TO_EXISTING_REPORT_FILE,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_append,
										INACTIVE, report_table, 0, 7, 1, 2);
	check_button3 = check_create_on_table(STR_LIST_ALL_SCANNED_FILES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_list,
										INACTIVE, report_table, 0, 7, 2, 3);

	frame7 = frame_create_on_table(STR_OPTIONS, table1, 5, 10, 12, 20);
	option_table = table_create(5, 7, frame7, 2 , HOMOGENEUS);

	archives_button = check_create_on_table(STR_SCAN_INSIDE_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_archives_callback), ptr_flag_archives,
										ACTIVE, option_table, 0, 5, 0, 1);
	

	if (IS_NOT_SET(have_version)) {  /* have_version=0  f-prot version prior to 4.4.1 */
		check_button4 = check_create_on_table(STR_SCAN_COMPRESSED_EXECUTABLES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_comp_exe,
										ACTIVE, option_table, 0, 5, 1, 2);
	} else if (have_version >= 440 ) {  /* f-prot version 4.4.1  and above */
		/* -packed is default in f-prot 4.4.1 and above so we don't need a checkbox for it
		 as a nopacked options doesn't exist and we can use this place for this new option */
		passwd_archives_button = check_create_on_table( STR_SCAN_PROTECTED_ARCHIVES,
										GTK_SIGNAL_FUNC(flag_passwd_archives_callback), ptr_flag_passwd_archives,
										INACTIVE, option_table, 0, 5, 1, 2);
	}
	
	check_button5 = check_create_on_table(STR_SCAN_FOR_OTHER_VARIOUS_MALWARE,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_collect,
										INACTIVE, option_table, 0, 5, 2, 3);
	ai_check_button = check_create_on_table(STR_ENABLE_NEURAL_NETWORK_VIRUS_DETECTION,
										GTK_SIGNAL_FUNC(flag_ai_callback), ptr_flag_ai,
										INACTIVE, option_table, 0, 5, 3, 4);
	check_button6 = check_create_on_table(STR_FOLLOW_SYMBOLIC_LINKS,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_follow,
										INACTIVE, option_table, 0, 5, 4, 5);
	check_button7 = check_create_on_table(STR_DO_NOT_COMPLAIN_ABOUT_OLD_DEF_FILES,
										GTK_SIGNAL_FUNC(flag_callback), ptr_flag_old,
										INACTIVE, option_table, 0, 5, 5, 6);

	label = label_create_on_table( STR_USE_HEURISTICS, option_table, 0, 2, 6,7);
	heur_1_button = radio_create_on_table(NULL, STR_YES,GTK_SIGNAL_FUNC(flag_heur_callback), "0",
										 ACTIVE, option_table, 2, 3, 6, 7);
	heuristics_group = get_radio_button_group (GTK_RADIO_BUTTON (heur_1_button));
	heur_2_button = radio_create_on_table(heur_1_button, STR_NO,GTK_SIGNAL_FUNC(flag_heur_callback), "1",
										INACTIVE, option_table, 3, 4, 6, 7);
	heur_3_button = radio_create_on_table(heur_2_button, STR_ONLY,GTK_SIGNAL_FUNC(flag_heur_callback), "2",
										INACTIVE, 	option_table, 4, 5, 6, 7);
	
	onlyheur_tooltip=gtk_tooltips_new();
	#ifndef USE_GTK_2_0
	/* Warning: Gtk+ 1.2.10 docs for gtk_tooltips_set_delay say:
	 * delay: an integer value representing seconds !!!
	 * but in gtktooltips.c  we read #define DEFAULT_DELAY 500   (Default delay in ms)
	 * So this seems to works for me only using seconds*1000
	 */
	gtk_tooltips_set_delay (onlyheur_tooltip,2000); /* 2 seconds */
	#endif
	gtk_tooltips_set_tip (GTK_TOOLTIPS (onlyheur_tooltip),heur_3_button,STR_NOHEUR_TOOLTIP,NULL);
	
	#ifdef USE_GTK_2_0 
	button3 = button_create_on_table(STR_INFO		,GTK_SIGNAL_FUNC(fprot_info)	, NULL, table1, 7, 8,  20, 21);
	button4 = button_create_on_table(STR_VIRUS_LIST	,GTK_SIGNAL_FUNC(fprot_virlist)	, NULL, table1, 6, 7,  20, 21);
	button5 = button_create_on_table(STR_TEST		,GTK_SIGNAL_FUNC(fprot_test)	, NULL, table1, 5, 6,  20, 21);
	button6 = button_create_on_table(STR_UPDATE		,GTK_SIGNAL_FUNC(fprot_update)	, NULL, table1, 4, 5,  20, 21);
	button7 = button_create_on_table(STR_REPORT		,GTK_SIGNAL_FUNC(view_report)	, NULL, table1, 3, 4,  20, 21);
	button8 = button_create_on_table(STR_SCAN		,GTK_SIGNAL_FUNC(scan_f)		, NULL, table1, 2, 3,  20, 21);
	#else
	/* with Gtk+1.2.x this looks better */
	button3 = button_create_on_table(STR_INFO		,GTK_SIGNAL_FUNC(fprot_info)	, NULL, table1, 8, 10,  20, 21);
	button4 = button_create_on_table(STR_TEST		,GTK_SIGNAL_FUNC(fprot_test)	, NULL, table1, 6,  8,  20, 21);
	button5 = button_create_on_table(STR_UPDATE		,GTK_SIGNAL_FUNC(fprot_update)	, NULL, table1, 4,  6,  20, 21);
	button6 = button_create_on_table(STR_REPORT		,GTK_SIGNAL_FUNC(view_report)	, NULL, table1, 2,  4,  20, 21);
	button7 = button_create_on_table(STR_SCAN		,GTK_SIGNAL_FUNC(scan_f)		, NULL, table1, 0,  2,  20, 21);	
	#endif
	
	gtk_widget_show_all(main_window);
	toggle1_f();
	toggle2_f();

	while (gtk_events_pending ())
		gtk_main_iteration();	
	
	/* display license window, this can be disabled */
	if (lstat(NO_SPLASH_FILE,&stat_p)!=0) {
		fd = open_file(XFPROT_BIN_DIR LICENSE_FILE);
		if (about_window_create( VERSION, 620, 430,8,
								VERSION"\n"COPYRIGHT"\n"STR_LICENSE"\n"HOMEPAGE"\n"STR_WARRANTY,
								fd,FIRST_TIME)==1) {
			create_file(NO_SPLASH_FILE);
		}
		xfclose_nostdin(fd);
	}
	#ifdef DEBUG
	else
		err_msg("NO_SPLASH_FILE");
	#endif
	
	main_window=main_loop(main_window);
	exit(EXIT_SUCCESS);
}
